@@ -5,6 +5,7 @@
   SPDX-FileCopyrightText: 2000-2004 Cornelius Schumacher <schumacher@kde.org>
   SPDX-FileCopyrightText: 2003-2004 Reinhold Kainhofer <reinhold@kainhofer.com>
   SPDX-FileCopyrightText: 2006 David Jarvie <djarvie@kde.org>
+  SPDX-FileCopyrightText: 2021 Boris Shmarin <b.shmarin@omp.ru>
 
   SPDX-License-Identifier: LGPL-2.0-or-later
 */
@@ -30,13 +31,13 @@
 
 #include "kcalendarcore_debug.h"
 
-#include <QTimeZone>
 
 extern "C" {
-#include <icaltimezone.h>
+#include <libical/icaltimezone.h>
 }
 
-#include <algorithm> // for std::remove()
+#include <algorithm>
+#include <set>
 
 using namespace KCalendarCore;
 
@@ -167,8 +168,11 @@ Person Calendar::owner() const
 
 void Calendar::setOwner(const Person &owner)
 {
-    d->mOwner = owner;
-    setModified(true);
+    if (owner != d->mOwner) {
+        d->mOwner = owner;
+        setModified(true);
+        Q_EMIT ownerChanged();
+    }
 }
 
 void Calendar::setTimeZone(const QTimeZone &timeZone)
@@ -217,7 +221,8 @@ void Calendar::shiftTimes(const QTimeZone &oldZone, const QTimeZone &newZone)
 {
     setTimeZone(newZone);
 
-    int i, end;
+    int i;
+    int end;
     Event::List ev = events();
     for (i = 0, end = ev.count(); i < end; ++i) {
         ev[i]->shiftTimes(oldZone, newZone);
@@ -251,19 +256,20 @@ CalFilter *Calendar::filter() const
 
 QStringList Calendar::categories() const
 {
-    Incidence::List rawInc(rawIncidences());
-    QStringList cats, thisCats;
+    const Incidence::List rawInc = rawIncidences();
+    QStringList uniqueCategories;
+    QStringList thisCats;
     // @TODO: For now just iterate over all incidences. In the future,
     // the list of categories should be built when reading the file.
-    for (Incidence::List::ConstIterator i = rawInc.constBegin(); i != rawInc.constEnd(); ++i) {
-        thisCats = (*i)->categories();
-        for (QStringList::ConstIterator si = thisCats.constBegin(); si != thisCats.constEnd(); ++si) {
-            if (!cats.contains(*si)) {
-                cats.append(*si);
+    for (const Incidence::Ptr &inc : rawInc) {
+        thisCats = inc->categories();
+        for (const auto &cat : std::as_const(thisCats)) {
+            if (!uniqueCategories.contains(cat)) {
+                uniqueCategories.append(cat);
             }
         }
     }
-    return cats;
+    return uniqueCategories;
 }
 
 Incidence::List Calendar::incidences(const QDate &date) const
@@ -303,20 +309,17 @@ Incidence::List Calendar::instances(const Incidence::Ptr &incidence) const
 
 Incidence::List Calendar::duplicates(const Incidence::Ptr &incidence)
 {
-    if (incidence) {
-        Incidence::List list;
-        Incidence::List vals = values(d->mNotebookIncidences);
-        Incidence::List::const_iterator it;
-        for (it = vals.constBegin(); it != vals.constEnd(); ++it) {
-            if (((incidence->dtStart() == (*it)->dtStart()) || (!incidence->dtStart().isValid() && !(*it)->dtStart().isValid()))
-                && (incidence->summary() == (*it)->summary())) {
-                list.append(*it);
-            }
-        }
-        return list;
-    } else {
-        return Incidence::List();
+    if (!incidence) {
+        return {};
     }
+
+    Incidence::List list;
+    const Incidence::List vals = values(d->mNotebookIncidences);
+    std::copy_if(vals.cbegin(), vals.cend(), std::back_inserter(list), [&](const Incidence::Ptr &in) {
+        return (incidence->dtStart() == in->dtStart() || (!incidence->dtStart().isValid() && !in->dtStart().isValid()))
+            && incidence->summary() == in->summary();
+    });
+    return list;
 }
 
 bool Calendar::addNotebook(const QString &notebook, bool isVisible)
@@ -335,11 +338,12 @@ bool Calendar::updateNotebook(const QString &notebook, bool isVisible)
         return false;
     } else {
         d->mNotebooks.insert(notebook, isVisible);
-        const QList<Incidence::Ptr> incidences = d->mNotebookIncidences.values(notebook);
-        for (Incidence::Ptr incidence : incidences) {
-            QHash<Incidence::Ptr, bool>::Iterator it = d->mIncidenceVisibility.find(incidence);
-            if (it != d->mIncidenceVisibility.end())
-                *it = isVisible;
+        for (auto noteIt = d->mNotebookIncidences.cbegin(); noteIt != d->mNotebookIncidences.cend(); ++noteIt) {
+            const Incidence::Ptr &incidence = noteIt.value();
+            auto visibleIt = d->mIncidenceVisibility.find(incidence);
+            if (visibleIt != d->mIncidenceVisibility.end()) {
+                *visibleIt = isVisible;
+            }
         }
         return true;
     }
@@ -423,11 +427,10 @@ bool Calendar::setNotebook(const Incidence::Ptr &inc, const QString &notebook)
                 return false;
             }
             // Move all possible children also.
-            Incidence::List list = instances(inc);
-            Incidence::List::Iterator it;
-            for (it = list.begin(); it != list.end(); ++it) {
-                d->mNotebookIncidences.remove(old, *it);
-                d->mNotebookIncidences.insert(notebook, *it);
+            const Incidence::List list = instances(inc);
+            for (const auto &incidence : list) {
+                d->mNotebookIncidences.remove(old, incidence);
+                d->mNotebookIncidences.insert(notebook, incidence);
             }
             notifyIncidenceChanged(inc); // for removing from old notebook
             // don not remove from mUidToNotebook to keep deleted incidences
@@ -578,7 +581,9 @@ Incidence::Ptr Calendar::createException(const Incidence::Ptr &incidence, const
     }
 
     Incidence::Ptr newInc(incidence->clone());
-    newInc->setCreated(QDateTime::currentDateTimeUtc());
+    const QDateTime current = QDateTime::currentDateTimeUtc();
+    newInc->setCreated(current);
+    newInc->setLastModified(current);
     newInc->setRevision(0);
     // Recurring exceptions are not support for now
     newInc->clearRecurrence();
@@ -639,27 +644,21 @@ Incidence::List Calendar::incidencesFromSchedulingID(const QString &sid) const
 {
     Incidence::List result;
     const Incidence::List incidences = rawIncidences();
-    Incidence::List::const_iterator it = incidences.begin();
-    for (; it != incidences.end(); ++it) {
-        if ((*it)->schedulingID() == sid) {
-            result.append(*it);
-        }
-    }
+    std::copy_if(incidences.cbegin(), incidences.cend(), std::back_inserter(result), [&sid](const Incidence::Ptr &in) {
+        return in->schedulingID() == sid;
+    });
     return result;
 }
 
 Incidence::Ptr Calendar::incidenceFromSchedulingID(const QString &uid) const
 {
     const Incidence::List incidences = rawIncidences();
-    Incidence::List::const_iterator it = incidences.begin();
-    for (; it != incidences.end(); ++it) {
-        if ((*it)->schedulingID() == uid) {
-            // Touchdown, and the crowd goes wild
-            return *it;
-        }
-    }
-    // Not found
-    return Incidence::Ptr();
+    const auto itEnd = incidences.cend();
+    auto it = std::find_if(incidences.cbegin(), itEnd, [&uid](const Incidence::Ptr &in) {
+        return in->schedulingID() == uid;
+    });
+
+    return it != itEnd ? *it : Incidence::Ptr();
 }
 
 /** static */
@@ -728,6 +727,14 @@ Todo::List Calendar::sortTodos(const Todo::List &todoList, TodoSortField sortFie
             std::sort(todoListSorted.begin(), todoListSorted.end(), Todos::createdMoreThan);
         }
         break;
+
+    case TodoSortCategories:
+        if (sortDirection == SortDirectionAscending) {
+            std::sort(todoListSorted.begin(), todoListSorted.end(), Incidences::categoriesLessThan);
+        } else {
+            std::sort(todoListSorted.begin(), todoListSorted.end(), Incidences::categoriesMoreThan);
+        }
+        break;
     }
 
     return todoListSorted;
@@ -824,7 +831,7 @@ void Calendar::setupRelations(const Incidence::Ptr &forincidence)
     }
 
     // Now see about this incidences parent
-    if (forincidence->relatedTo().isEmpty() && !forincidence->relatedTo().isEmpty()) {
+    if (!forincidence->relatedTo().isEmpty()) {
         // Incidence has a uid it is related to but is not registered to it yet.
         // Try to find it
         Incidence::Ptr parent = incidence(forincidence->relatedTo());
@@ -881,8 +888,8 @@ void Calendar::removeRelations(const Incidence::Ptr &incidence)
         // Since the mOrphans dict might contain the same key (with different
         // child incidence pointers!) multiple times, take care that we remove
         // the correct one. So we need to remove all items with the given
-        // parent UID, and readd those that are not for this item. Also, there
-        // might be other entries with differnet UID that point to this
+        // parent UID, and re-add those that are not for this item. Also, there
+        // might be other entries with different UID that point to this
         // incidence (this might happen when the relatedTo of the item is
         // changed before its parent is inserted. This might happen with
         // groupware servers....). Remove them, too
@@ -891,26 +898,22 @@ void Calendar::removeRelations(const Incidence::Ptr &incidence)
         // First, create a list of all keys in the mOrphans list which point
         // to the removed item
         relatedToUids << incidence->relatedTo();
-        for (QMultiHash<QString, Incidence::Ptr>::Iterator it = d->mOrphans.begin(); it != d->mOrphans.end(); ++it) {
+        for (auto it = d->mOrphans.cbegin(); it != d->mOrphans.cend(); ++it) {
             if (it.value()->uid() == uid) {
                 relatedToUids << it.key();
             }
         }
 
         // now go through all uids that have one entry that point to the incidence
-        for (QStringList::const_iterator uidit = relatedToUids.constBegin(); uidit != relatedToUids.constEnd(); ++uidit) {
-            Incidence::List tempList;
+        for (const auto &relUid : std::as_const(relatedToUids)) {
             // Remove all to get access to the remaining entries
-            const Incidence::List l = values(d->mOrphans, *uidit);
-            d->mOrphans.remove(*uidit);
-            for (const Incidence::Ptr &i : l) {
-                if (i != incidence) {
-                    tempList.append(i);
-                }
-            }
-            // Readd those that point to a different orphan incidence
-            for (Incidence::List::Iterator incit = tempList.begin(); incit != tempList.end(); ++incit) {
-                d->mOrphans.insert(*uidit, *incit);
+            Incidence::List lst = values(d->mOrphans, relUid);
+            d->mOrphans.remove(relUid);
+            lst.erase(std::remove(lst.begin(), lst.end(), incidence), lst.end());
+
+            // Re-add those that point to a different orphan incidence
+            for (const auto &in : std::as_const(lst)) {
+                d->mOrphans.insert(relUid, in);
             }
         }
     }
@@ -1161,7 +1164,8 @@ Incidence::List Calendar::mergeIncidenceList(const Event::List &events, const To
     Incidence::List incidences;
     incidences.reserve(events.count() + todos.count() + journals.count());
 
-    int i, end;
+    int i;
+    int end;
     for (i = 0, end = events.count(); i < end; ++i) {
         incidences.append(events[i]);
     }
@@ -1254,7 +1258,7 @@ void Calendar::appendRecurringAlarms(Alarm::List &alarms, const Incidence::Ptr &
                 }
 
                 // Adjust the 'alarmStart' date/time and find the next recurrence at or after it.
-                // Treate the two offsets separately in case one is daily and the other not.
+                // Treat the two offsets separately in case one is daily and the other not.
                 dt = incidence->recurrence()->getNextDateTime(baseStart.addSecs(-1));
                 if (!dt.isValid() || (dt = endOffset.end(offset.end(dt))) > to) { // adjust 'dt' to get the alarm time
                     // The next recurrence is too late.
@@ -1345,3 +1349,53 @@ void Calendar::virtual_hook(int id, void *data)
     Q_UNUSED(data);
     Q_ASSERT(false);
 }
+
+QString Calendar::id() const
+{
+    return d->mId;
+}
+
+void Calendar::setId(const QString &id)
+{
+    if (d->mId != id) {
+        d->mId = id;
+        Q_EMIT idChanged();
+    }
+}
+
+QString Calendar::name() const
+{
+    return d->mName;
+}
+
+void Calendar::setName(const QString &name)
+{
+    if (d->mName != name) {
+        d->mName = name;
+        Q_EMIT nameChanged();
+    }
+}
+
+QIcon Calendar::icon() const
+{
+    return d->mIcon;
+}
+
+void Calendar::setIcon(const QIcon &icon)
+{
+    d->mIcon = icon;
+    Q_EMIT iconChanged();
+}
+
+AccessMode Calendar::accessMode() const
+{
+    return d->mAccessMode;
+}
+
+void Calendar::setAccessMode(const AccessMode mode)
+{
+    if (d->mAccessMode != mode) {
+        d->mAccessMode = mode;
+        Q_EMIT accessModeChanged();
+    }
+}
